/*
 * Asterisk -- A telephony toolkit for Linux.
 *
 * ReadDigits --- read some DTMF digits and put them in a variable
 * 
 * Copyright (C) 2003, NTG Clarity Networks Inc.
 *
 * William Waites <ww@ntgclarity.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 */
 
#include <asterisk/lock.h>
#include <asterisk/file.h>
#include <asterisk/logger.h>
#include <asterisk/channel.h>
#include <asterisk/pbx.h>
#include <asterisk/module.h>
#include <asterisk/translate.h>
#include <asterisk/ulaw.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <pthread.h>
#include <sys/time.h>

#define	MAX_DIGITS 20

static char *tdesc = "ReadDigits -- read some digits, terminated by #";

static char *app = "ReadDigits";

static char *synopsis = "ReadDigits(max) -- read some digits, terminated by #";

static char *descrip = 
        "ReadDigits -- Read some DTMF digits, terminated by # and place them in a variable\n"
        "              called ${DIGITS}\n\n"
        "It takes an optional parameter, max, which is a number specifying the maximum number\n"
        "of digits to read, and defaults to 20\n";

STANDARD_LOCAL_USER;

LOCAL_USER_DECL;

static int firstdigittimeout = 20000; /* 20 seconds first digit timeout */
static int digittimeout = 10000; /* 10 seconds subsequent digit timeout */

static int ms_diff(struct timeval *tv1, struct timeval *tv2)
{
  int	ms;
  ms = (tv1->tv_sec - tv2->tv_sec) * 1000;
  ms += (tv1->tv_usec - tv2->tv_usec) / 1000;
  return(ms);
}

static int readdigits_exec(struct ast_channel *chan, void *data)
{
  int i,j; /* digit index, temporary digit holding variable */
  int maxdigits = MAX_DIGITS;
  char *args = data;
  char digits[MAX_DIGITS+1];

  struct localuser *u;

  struct ast_frame *f;
  struct timeval lastout, now, lastdigittime;
  int res;

  lastout.tv_sec = lastout.tv_usec = 0;

  memset(digits, 0, sizeof(digits));

  if(*args && (sscanf(args, "%d", &maxdigits) != 1)) {
    ast_log(LOG_WARNING, "ReadDigits: garbled parameter: %s shoud be a number\n", args);
    return -1;
  }

  if (maxdigits > MAX_DIGITS)
    maxdigits = MAX_DIGITS;

  LOCAL_USER_ADD(u);

  if (chan->_state != AST_STATE_UP) {
    /* answer */
    ast_answer(chan);
  }

  i = j = 0;

  gettimeofday(&lastdigittime,NULL);
  for(;;) {
    gettimeofday(&now,NULL);
    /* if outa time, goto priority n+101 */
    if (ms_diff(&now,&lastdigittime) > ((j) ? digittimeout : firstdigittimeout)) {
      ast_log(LOG_DEBUG,"ReadDigits: digit entry timeout on chan %s\n", chan->name);
      LOCAL_USER_REMOVE(u);
      chan->priority += 100;
      return -1;
    }
    if ((res = ast_waitfor(chan, -1) < 0)) {
      ast_log(LOG_DEBUG, "Waitfor returned %d\n", res);
      continue;
    }
			
    f = ast_read(chan);
    if (!f) {
      ast_log(LOG_DEBUG, "ReadDigits: error reading frame\n");
      LOCAL_USER_REMOVE(u);
      return -1;
    }
    if ((f->frametype == AST_FRAME_CONTROL) && (f->subclass == AST_CONTROL_HANGUP)) {
      ast_frfree(f);
      ast_log(LOG_DEBUG, "ReadDigits: got Hangup\n");
      LOCAL_USER_REMOVE(u);
      return -1;
    }
  
    /* if not DTMF, just do it again */
    if (f->frametype != AST_FRAME_DTMF) {
      ast_frfree(f);
      continue;
    }
  
    /* got a DTMF tone */

    j = f->subclass;  /* save digit */
    ast_frfree(f);

    ast_log(LOG_DEBUG, "ReadDigits: read %c\n", j);

    if (j == '#') /* end input on # */
      break;
		
    digits[i++] = j;

    if (i == maxdigits) { /* if still valid number of digits */
      ast_log(LOG_DEBUG, "ReadDigits: read maximum number of digits (%d)\n", maxdigits);
      break;
    }

    gettimeofday(&lastdigittime,NULL);
  }
	
  pbx_builtin_setvar_helper(chan, "DIGITS", digits);

  LOCAL_USER_REMOVE(u);
  return 0;
}

int unload_module(void)
{
  STANDARD_HANGUP_LOCALUSERS;
  return ast_unregister_application(app);
}

int load_module(void)
{
  return ast_register_application(app, readdigits_exec, synopsis, descrip);
}

char *description(void)
{
  return tdesc;
}

int usecount(void)
{
  int res;
  STANDARD_USECOUNT(res);
  return res;
}

char *key(void)
{
  return ASTERISK_GPL_KEY;
}
